define(['jquery', 'sandbox'], function($, sandbox) {

  var Request = sandbox.mvc.Model({

    defaults: {
      sync_status: 'idle',
      status: 'pending'
    },

    url: 'services/certificates/api/v1/csrs/',
    cacheUrl: 'services/certificates/api/v1/cache/',

    idAttribute: 'name',

    /**
     * Accept the certificate signing request
     * @return {jQuery.promise}
     */
    accept: function() {
      return this._sync_status('PUT', $.proxy(this.set_accepted, this));
    },

    /**
     * Reject the certificate signing request
     * @return {jQuery.promise}
     */
    reject: function () {
      return this._sync_status('DELETE', $.proxy(this.set_rejected, this));
    },

    /**
     *
     * @param request_method {string}
     * @param model_callback {Function}
     * @return {jQuery.promise}
     * @private
     */
    _sync_status: function(request_method, model_callback) {
      var deferred = new sandbox.data.deferred();

      deferred.done([
        model_callback,
        $.proxy(this.set_sync_complete, this)
      ]);

      deferred.fail([
        $.proxy(this.set_failed, this),
        $.proxy(this.set_sync_failed, this)
      ]);

      this.set_sync_saving();

      $.ajax({
        url: this.url + this.id,
        type: request_method,

        success: function() {
          deferred.resolve();
        },

        error: function(resp) {
          deferred.reject(resp.responseText);
        }
      });

      return deferred.promise();
    },

    set_accepted: function() {
      this.set({
        request_open: false,
        status: 'accepted'
      })
    },

    set_rejected: function() {
      this.set({
        request_open: false,
        status: 'rejected'
      })
    },

    set_failed: function(reason) {
      this.set({
        message: reason,
        status: 'pending'
      })
    },

    /**
     * As signing requests don't corrospond with a RESTy model, we don't want to
     * call the server on model creation/destruction, so we override the sync() method to a noop.
     */
    sync: function() {},

    set_sync_idle: function() {
      this.trigger('sync:idle');
      this.set('sync_status', 'idle');
    },

    set_sync_saving: function() {
      this.trigger('sync:saving');
      this.set('sync_status', 'saving');
    },

    set_sync_complete: function() {
      this.trigger('sync:complete');
      this.set('sync_status', 'complete');
    },

    set_sync_failed: function() {
      this.trigger('sync:failed');
      this.set('sync_status', 'failed');
    }

  });

  return Request;

});
